//
// bld_demo.c
// testing blending
//
// (20031013 - 20060926, cearn)
//   bg 0, cbb  0, sbb  1, pb 1: bg metroid
//   bg 1, cbb  1, sbb  2, pb 0: text / fence
//   oam 0: tile 0-63: obj metroid

#include <stdio.h>
#include <tonc.h>
#include "metr.h"

void test_blend()
{
	char str[64];
	se_puts( 48, 8, "obj", 0);
	se_puts(168, 8, "bg", 0);

	int mode=0;
	// eva, evb and ey are .4 fixeds
	// eva is full, evb and ey are empty
	u32 eva=0x80, evb= 0, ey=0;

	REG_BLDCNT= BLD_BUILD(
		BLD_OBJ | BLD_BG0,	// Top
		BLD_BG1,			// Bottom
		mode);				// Mode
	// also try this variation once:
	//BLD_BUILD(BLD_OBJ | BLD_BG0 |BLD_BG1, BLD_BG1 | BLD_BD, mode<<6);

	while(1)
	{
		vid_vsync();
		key_poll();

		// interactive blend weights
		eva += key_tri_horz();
		evb -= key_tri_vert();
		ey  += key_tri_fire();
		
		mode += bit_tribool(key_hit(-1), KI_R, KI_L);

		// clamp to allowable ranges
		eva = clamp(eva, 0, 0x81);
		evb = clamp(evb, 0, 0x81);
		ey  = clamp(ey , 0, 0x81);
		mode= clamp(mode, 0, 4);

		siprintf(str, "mode: %2d\n eva: %2d\n evb: %2d\n  ey: %2d", 
			mode, eva/8, evb/8, ey/8);
		se_puts(80, 96, str, 0);
		
		// Update blend mode
		BF_SET(REG_BLDCNT, mode, BLD_MODE);

		// Update blend weights
		REG_BLDALPHA= BLDA_BUILD(eva/8, evb/8);
		REG_BLDY= BLDY_BUILD(ey/8);
	}	
}

void load_metr()
{
	// copy sprite and bg tiles, and the sprite palette
	memcpy32(&tile_mem[0][0], metrTiles, metrTilesLen/4);
	memcpy32(&tile_mem[4][0], metrTiles, metrTilesLen/4);
	memcpy32(pal_obj_mem, metrPal, metrPalLen/4);

	// set the metroid sprite
	OBJ_ATTR *metr= &oam_mem[0];	// use the first sprite
	obj_set_attr(metr, ATTR0_SQUARE | ATTR0_BLEND, ATTR1_SIZE_64, 0);
	obj_set_pos(metr, 32, 24);		// mid-center

	// create the metroid bg
	// using inverted palette for bg-metroid
	int ix, iy;
	for(ix=0; ix<16; ix++)
		pal_bg_mem[ix+16]= pal_obj_mem[ix] ^ CLR_WHITE;

	SCR_ENTRY *pse= &se_mem[1][3*32+18];	// right-center
	for(iy=0; iy<8; iy++)
		for(ix=0; ix<8; ix++)
			pse[iy*32+ix]= iy*8+ix + SE_PALBANK(1);

	REG_BG0CNT= BG_CBB(0) | BG_SBB(1);
}

// set-up the fence/text background
void load_fence()
{
	txt_init_std();
	txt_init_se(1, BG_CBB(1)|BG_SBB(2), 0, CLR_ORANGE, 0xEE);

	// tile 0 / ' ' will be a fence tile
	const TILE fence= 
	{{ 
		0x00012000, 0x00012000, 0x00022200, 0x22220222,
		0x11122211, 0x00112000, 0x00012000, 0x00012000,
	}};
	tile_mem[1][0]= fence;

	pal_bg_mem[0]= RGB15(16, 10, 20);
	pal_bg_mem[1]= RGB15( 0,  0, 31);
	pal_bg_mem[2]= RGB15(16, 16, 16);
}

int main()
{
	oam_init(oam_mem, 128);
	load_metr();
	load_fence();

	REG_DISPCNT= DCNT_BUILD(0,1+2+16,0,1,0);

	test_blend();

	return 0;
}
